"""Auth/z middleware and Login/Logout handlers."""
import logging
import pprint
import time
import uuid
from typing import Tuple

import diskcache
import jwkest
from falcon import HTTPFound, HTTPUnauthorized, Request, Response
from oic.oauth2.grant import Token
from oic.oic import AuthorizationResponse

from src.settings import (OSDU_AUTH_LOGOUT_URL, OSDU_AUTH_REDIRECT_URL,
                          OSDU_TARGET_PLATFORM)

from .client import oidc_client, scopes
from .session import Session

logger = logging.getLogger(__name__)


def get_session_from_request(request: Request, disk_cache: diskcache.Cache) -> Session:
    """Get session cookie from request header."""
    session_cookie = request.get_cookie_values("osdu_training_session_id")
    logger.debug(
        f"Found session cookie in request header: {session_cookie}")

    # verify the session cookie exists
    if session_cookie is None:
        description = "No session id found, please sign-in first."
        logger.debug(description)
        raise HTTPUnauthorized("Sign-in required", description)

    # get session id from session cookie
    session_id = session_cookie[0]
    # verify if session object is still in cache
    if session_id not in disk_cache:
        disk_cache_keys = list(disk_cache.iterkeys())
        logger.debug(
            f"Cannot find session object for session id. Keys exist: {disk_cache_keys}")
        description = "Invalid session id, please sign-in again."
        raise HTTPUnauthorized("Sign-in required", description)

    # get session object from disk cache
    return disk_cache.get(session_id)


def preprocessing_auth_callback_response(
        request: Request,
        disk_cache: diskcache.Cache
) -> Tuple[Session, object]:
    """Preprocessing auth provider callback response."""
    # get session cookie from request header
    session = get_session_from_request(request, disk_cache)

    # processing callback response
    callback_response = oidc_client.parse_response(
        AuthorizationResponse,
        info=request.query_string,
        sformat="urlencoded")
    logger.debug(
        f"Post request assertion (response, session):\
        {callback_response['state']}, {session.state}")

    # Note that the simple session disk_cache assumes one client.
    # In other words, this may assert if two browsers point at your server!
    if not callback_response["state"] == session.state:
        description = "Session status not accepted"
        logger.debug(description)
        raise HTTPUnauthorized("Sign-in required", description)

    return session, callback_response


class AuthMiddleware:
    """Implements session management and caching its content to a disk cache."""

    def __init__(self, disk_cache):
        """Inject the disk cache object to be used for sessions caching."""
        self.disk_cache = disk_cache

    # executed AFTER routing, BEFORE invoking the resource handler
    def process_resource(self, request, _response, resource, _param):
        """Verify if the cookie is present and puts session object to request context."""
        logger.debug(f"Handling authentication for resource: {resource}")

        # skip authentication for Login and Callback handlers
        if isinstance(resource, (AuthCallbackResource, LoginResource)):
            logger.debug("Skipping auth middleware for this resource...")
            return

        # get session cookie from request header
        session = get_session_from_request(request, self.disk_cache)
        logger.debug(f"Found session object in memory cache: {session}")

        # check session state
        if session.state == "training/login":
            description = f"Authorization not completed. Ensure the\
                'Preload pages for faster browsing and searching'\
                option is disabled in the browser privacy settings and retry logging in"
            logger.debug(description)
            raise HTTPUnauthorized("Sign-in required", description)

        # decode jwt token (id_token)
        jwt = jwkest.jws.JWT()
        jwt.unpack(session.id_token)
        # check id token expiration time and refresh if needed
        if jwt.payload()["exp"] < time.time():
            self.refresh_tokens(session.session_id, session)

        # add session object to request context
        request.context.session = session

    def refresh_tokens(self, session_id, session):
        """Renews `id` and `access` tokens if valid `refresh` token is present."""
        # make id/access tokens renew request using refresh token
        logger.debug("Renewing id and access tokens using refresh token")
        token = Token({"refresh_token": session.refresh_token})
        refresh_tokes_response = oidc_client.do_access_token_refresh(
            token=token)

        # update session object with new tokens
        session.id_token = refresh_tokes_response.raw_id_token
        session.access_token = refresh_tokes_response["access_token"]
        self.disk_cache.set(session_id, session)


class LoginResource:
    """Implements user sign-in process."""

    def __init__(self, disk_cache):
        """Login initialization and set disk cache attribute."""
        self.disk_cache = disk_cache

    def on_get(self, request: Request, response: Response) -> None:
        """
        Login handler to initiate the sign-in process.

        Initiates the sign-in process by constructing the
        authorization request and redirecting to the provider
        authorization endpoint.
        """
        # get frontend state before sign-in and save it to redirect back
        # to this page after user login is complete
        frontend_state = request.params.get("frontend_state")

        # generate new session id using uuid4 (random)
        session_id = str(uuid.uuid4())

        # create new session object with id and state (parameter specific)
        if frontend_state is None:
            session = Session(session_id, "training/login")
        else:
            session = Session(session_id, frontend_state)

        # store session object in disk cache
        self.disk_cache.set(session_id, session)

        # put session id to cookie as we don't want to pass
        # the session object back to the browser since it will
        # contain sensitive data such as auth, id and refresh tokens
        response.set_cookie("osdu_training_session_id", session_id)
        logger.debug(f"Setting cookies for session: {session}")
        logger.debug(
            f"Initiating user sign-in process for session: {session_id}")

        # set parameters for authorization server request.
        # 'select_account' value prompts the End-User to select a user account,
        # is used for better user experience during a login process
        args = {
            "response_type": "code",
            "client_id": oidc_client.client_id,
            "scope": scopes,
            "redirect_uri": OSDU_AUTH_REDIRECT_URL,
            "state": session.state,
            "prompt": "select_account",
        }

        # ==> BEGIN: provider-specific code <==
        # GCP requires specific request parameter
        # to get the refresh token.
        if OSDU_TARGET_PLATFORM == "gcp":
            args["access_type"] = "offline"
        # <== END: provider-specific code ==>

        # create authorization request
        auth_request = oidc_client.construct_AuthorizationRequest(
            request_args=args)
        logger.debug(f"Creating authorization request:\
        (clientID: {auth_request['client_id']},\
            scopes: {auth_request['scope']},\
            RedirectURL: {auth_request['redirect_uri']})")

        # get login url from provider configuration
        login_url = auth_request.request(oidc_client.authorization_endpoint)

        # redirect to the authorization server:
        # DO NOT use HTTP 301 (falcon.HTTPMovedPermanently) since
        # most browsers will cache this url for future requests
        # effectively bypassing your portion of sign-in process
        logger.info(f"Redirecting to Login URL: {login_url}")
        raise HTTPFound(login_url)  # HTTP 302


class AuthCallbackResource:
    """Handles auth provider callback after sign-in process is complete."""

    def __init__(self, disk_cache):
        """Auth callback resource initialization."""
        self.disk_cache = disk_cache

    def on_get(self, request: Request, response: Response) -> None:
        """
        Auth callback handler to exchange the authorization code for the tokens.

        Validates the state, so it hasn't changed during the communication
        process and then exchanges `authorization code` for the `access_token`
        by calling the `do_access_token() method`.
        """
        # prepare session and callback response
        session, auth_response = preprocessing_auth_callback_response(
            request, self.disk_cache)

        # extract authorization code from auth server response
        args = {
            "code": auth_response["code"],
        }

        # exchange authorization token for the access token
        access_token_response = oidc_client.do_access_token_request(
            state=auth_response["state"],
            request_args=args
        )

        # update session information (tokens)
        session.state = "training/auth/callback"
        session.id_token = access_token_response.raw_id_token
        session.access_token = access_token_response["access_token"]
        session.refresh_token = access_token_response["refresh_token"]
        self.disk_cache.set(session.session_id, session)
        logger.debug(
            f"Adding tokens to memory cache for the session: {session}")

        # dump tokens back to browser if frontend_state not present
        # -- this is for DEMO only! DO NOT do this in production!
        # if frontend_state is present then redirect back to this url
        if auth_response["state"] == "training/login":
            response.body = pprint.pformat(access_token_response.__dict__)
        else:
            logger.debug(f"Redirecting back to {auth_response['state']}")
            raise HTTPFound(auth_response["state"])


class LogoutResource:
    """Implements user sign-out process."""

    def __init__(self, disk_cache):
        """Logout resource initialization."""
        self.disk_cache = disk_cache

    def on_get(self, request: Request, _response: Response) -> None:
        """
        Logout handler to destroy user session.

        Initiates the logout process by constructing the
        end session request and redirecting to the provider
        endpoint.
        """
        # get frontend state from request parameters
        # and check if it is present/valid
        frontend_state = request.params.get("frontend_state")
        if frontend_state is None:
            state = "training/logout"
        else:
            state = frontend_state

        # get session cookie from request header
        session = get_session_from_request(request, self.disk_cache)
        logger.debug(f"Found session object in memory cache: {session}")

        # configure parameters for sign-out request
        args = {
            "id_token": session.id_token,
            "post_logout_redirect_uri": OSDU_AUTH_LOGOUT_URL,
            "state": state,
        }

        # create logout request
        logout_request = oidc_client.construct_EndSessionRequest(
            request_args=args)
        logger.debug(f"Creating logout request:\
            state: {args['state']},\
            RedirectURL: {args['post_logout_redirect_uri']})")

        # get logout url
        logout_url = logout_request.request(oidc_client.end_session_endpoint)

        # fix oic client bug with id_token name (replace id_token on id_token_hint)
        logout_url = logout_url.replace("id_token", "id_token_hint")

        # update session state
        session.state = state
        self.disk_cache.set(session.session_id, session)
        logger.debug(f"Changing session state: {session}")

        # redirect to the authorization server:
        # DO NOT use HTTP 301 (falcon.HTTPMovedPermanently) since
        # most browsers will cache this url for future requests
        # effectively bypassing your portion of sign-out process
        logger.info(f"Redirecting to Logout URL: {logout_url}")
        raise HTTPFound(logout_url)  # HTTP 302


class EndSessionCallbackResource:
    """Handles auth provider callback and removes session info."""

    def __init__(self, disk_cache):
        """Session end callback resource initialization."""
        self.disk_cache = disk_cache

    def on_get(self, request: Request, response: Response) -> None:
        """Session end callback handler to remove session."""
        # prepare session and callback response
        session, end_session_response = preprocessing_auth_callback_response(
            request, self.disk_cache)

        # remove session after logout
        self.disk_cache.pop(session.session_id)
        logger.info(f"Remove session from memory cache: {session}")

        # if frontend_state is not a real state then pass
        # otherwise, redirect to this url
        if end_session_response["state"] == "training/logout":
            pass
        else:
            logger.debug(
                f"Redirecting back to {end_session_response['state']}")
            raise HTTPFound(end_session_response["state"])
