"""Module with falcon routing system and setting disk cache."""
import logging

import falcon
from diskcache import Cache

from .auth import (AuthCallbackResource, AuthMiddleware,
                   EndSessionCallbackResource, LoginResource, LogoutResource)
from .fetch import (FetchMarkersByWellIDsResource,
                    FetchTrajectoryByWellboreIDResource)
from .find import (FindWellboresByWellIDResource, FindWellsByNameResource,
                   FindWellsInPolygonResource)

logging.basicConfig(
    level=logging.DEBUG,
    format="%(asctime)s:%(levelname)s:%(message)s")

# Python disk-backed cache, works similar to Redis
disk_cache = Cache(".session_cache")

# adding auth middleware to all handlers
api = falcon.API(middleware=[AuthMiddleware(disk_cache)])
# this option turns OFF enforcing SSL for cookies
# which is ON by default. When ON, browser will remove
# any cookies when communicating over HTTP.
# Use for DEV environment ONLY. Turn ON for PROD!
api.resp_options.secure_cookies_by_default = False

# Authorization endpoints

# Login and auth callback endpoints, AuthMiddleware ignores them
api.add_route("/login", LoginResource(disk_cache))
api.add_route("/auth/callback", AuthCallbackResource(disk_cache))

# Logout and end session callback
api.add_route("/logout", LogoutResource(disk_cache))
api.add_route("/end_session/callback", EndSessionCallbackResource(disk_cache))

# Service endpoints

# Find wells by name
api.add_route("/find/wells_by_name", FindWellsByNameResource())

# Find wellbore by well_id
api.add_route("/find/wellbores_by_well_id", FindWellboresByWellIDResource())

# Find wells in polygon
api.add_route("/find/wells_in_polygon", FindWellsInPolygonResource())

# Fetch trajectories
api.add_route("/fetch/trajectory_by_wellbore_id",
              FetchTrajectoryByWellboreIDResource())

# Fetch markers
api.add_route("/fetch/markers_by_well_ids", FetchMarkersByWellIDsResource())
