"""Module implements fetching markers by well IDs."""
import logging
from dataclasses import asdict, dataclass
from json import dumps
from operator import attrgetter
from typing import Any, Dict, List

from falcon import HTTP_NOT_FOUND, HTTPError, Request, Response

from src.settings import OSDU_DATA_PARTITION
from .utils import call_search_api

logger = logging.getLogger(__name__)


@dataclass
class Wellbore:
    """Oversimplified Wellbore structure."""

    well_id: str
    wellbore_id: str
    wellbore_name: str


@dataclass
class Marker:
    """Formation top marker structure."""

    def __post_init__(self):
        """Post-constructor transformation (auto-executed after __init__)."""
        self.depth = float(self.depth)

    name: str
    depth: float


@dataclass
class Well:
    """Well structure enriched with the markers.

    Markers are typically tied back to Wellbores not Wells,
    so we choose the first Wellbore for each Well, pull the list
    of Markers for it and enrich the parent Well with this list.
    """

    well_id: str
    facility_name: str
    marker_depths: List[float]
    formations: List[str]


ResponseJSON = List[Well]


class FetchMarkersByWellIDsResource:
    """Resource to fetch and process markers data."""

    DEFAULT_ALIAS_NAME_TYPE_ID = f"{OSDU_DATA_PARTITION}:reference-data--AliasNameType:Borehole%20Code:"

    def on_get(self, request: Request, response: Response) -> None:
        """
        Fetch markers for wells with given well_ids.

        Response have format:
        [{
            well_id: <WellID>
            facility_name: <FacilityName>
            marker_depths: List[float]
            formations: List[str]
        }]
        """
        # get well_ids from request, such as "opendes:master-data--Well:8690:"
        well_ids = request.params.get("well_ids").split(",")

        # get wellbores using Search API
        search_results = self.get_wellbores(request, well_ids)
        # extract the list of wellbores, we'll need them later

        wellbores = self.extract_wellbores(request, search_results, well_ids)
        # also extract the list of wellbore ids
        wellbore_ids = self.extract_wellbore_ids(wellbores)

        # get markers using Search API
        # Note: you can get markers using a couple of different ways:
        #   1) extract them from the corresponding raw files associated
        #   with each markers (this is probably the safest way to do).
        #   In this case we would have used the Delivery API to get this file.
        #   2) get markers from the WellboreMarker resource record
        #   which is typically populated (unlike other work-product-components)
        #   during load type because the OSDU schema permits doing this for markers.
        #   This method is a little easier to implement, so we used it here!
        search_results = self.get_markers(request, wellbore_ids)

        # parse results and return results back to frontend
        markers_list = self.extract_markers(search_results, wellbores)
        response_body = self.prepare_response(wellbores, markers_list)
        response.body = dumps([asdict(well) for well in response_body])

    @staticmethod
    def get_wellbores(request: Request, well_ids: List[str]) -> Dict:
        """Find Wellbores that belong to Wells with given well_ids."""

        logger.info(
            f"Preparing calls to fetch markers for well_ids: {well_ids}")

        # the query we construct is using "OR" logic, so will find
        # all wellbores for all well ids in the list passed
        stringify_well_ids = [f"\"{well_id}:\"" for well_id in well_ids]
        query_params = " OR ".join(stringify_well_ids)
        search_query = {
            "kind": f"{OSDU_DATA_PARTITION}:wks:master-data--Wellbore:1.0.0",
            "query": f"data.WellID: ({query_params})",
            "returnedFields": [
                "id",
                "data.WellID",
                "data.NameAliases"
            ]
        }
        logger.debug(f"Search request payload: {search_query}")
        # make Search API call using utility function
        return call_search_api(request, search_query)

    def extract_wellbores(
            self,
            request: Request,
            search_results: Dict[str, list],
            well_ids: List[str]
    ) -> List[Wellbore]:
        """Extract wellbores from Search API response.
        'DEFAULT_ALIAS_NAME_TYPE_ID' 
        """
        well_ids_set = set(well_ids)
        wellbores = {}
        # find the first wellbore with that well_id
        for result in search_results["results"]:
            well_id = result["data"]["WellID"].rstrip(':')

            if well_id in well_ids_set:
                wellbore = Wellbore(
                    well_id=well_id,
                    wellbore_id=result["id"],
                    wellbore_name=self.extract_wellbore_name(result),
                )
                wellbores[wellbore.well_id] = wellbore
                well_ids_set.remove(wellbore.well_id)

        # check for well_ids without wellbores
        if well_ids_set:
            description = f"Wellbores don't exist for wells\
                            with these well_ids: {', '.join(well_ids_set)}"
            logger.exception(description)
            raise HTTPError(
                HTTP_NOT_FOUND,
                "Failed to extract wellbores",
                description)
        # return the list of wellbores objects extracted from search_results
        return [wellbores[well_id] for well_id in well_ids]

    @staticmethod
    def extract_wellbore_ids(wellbores: List[Wellbore]) -> List[str]:
        """Extract well_id from the list of Wellbores."""
        return [wellbore.wellbore_id for wellbore in wellbores]

    def extract_wellbore_name(self, record: Dict) -> str:
        """Extract wellbore name from the record aliases,
        DEFAULT_ALIAS_NAME_TYPE_ID attribute value is used as preferable alias type for wellbore name.
        """
        aliases = record["data"]["NameAliases"]

        for alias in aliases:
            if alias["AliasNameTypeID"] == self.DEFAULT_ALIAS_NAME_TYPE_ID:
                logger.debug(f"Found Wellbore name '{alias['AliasName']}'")
                return alias["AliasName"]
        else:
            logger.debug(
                f"No name for {self.DEFAULT_ALIAS_NAME_TYPE_ID:} alias for'{record}' record, "
                f"name '{aliases[0]['AliasName']}' will be used.")
            return aliases[0]["AliasName"]

    @staticmethod
    def get_markers(request: Request, wellbore_ids: List[str]) -> Dict[str, Any]:
        """Find Markers that belong to Wellbores with given wellbore_ids."""
        # the query we construct is using "OR" logic, so will find
        # all markers results for all wellbore ids in the list passed
        stringify_wellbore_ids = [
            f"\"{wellbore_id}:\"" for wellbore_id in wellbore_ids]
        query_params = " OR ".join(stringify_wellbore_ids)

        search_query = {
            "kind": f"{OSDU_DATA_PARTITION}:wks:work-product-component--WellboreMarkerSet:1.0.0",
            "query": f"data.WellboreID: ({query_params})",
            "returnedFields": [
                "data.WellboreID",
                "data.Markers",
            ]
        }

        # make Search API call using utility function
        return call_search_api(request, search_query)

    @staticmethod
    def extract_markers(
            search_results: Dict[str, Any],
            wellbores: List[Wellbore]
    ) -> List[List[Marker]]:
        """Extract list of Markers (which is also a list) from search results."""
        results = search_results["results"]
        markers_dict = {}

        for wellbore in results:
            markers = []
            for marker_data in wellbore["data"]["Markers"]:
                markers.append(
                    Marker(name=marker_data["MarkerName"],
                           depth=marker_data["MarkerMeasuredDepth"])
                )

            markers_dict[wellbore["data"]["WellboreID"].rstrip(":")] = markers

        return [markers_dict[wellbore.wellbore_id] for wellbore in wellbores]

    @staticmethod
    def prepare_response(
            wellbores: List[Wellbore],
            markers_list: List[List[Marker]]
    ) -> ResponseJSON:
        """Create a list of wells with the sorted Markers lists."""
        wells = []

        for wellbore, markers in zip(wellbores, markers_list):
            sorted_markers = sorted(markers, key=attrgetter("depth"))

            well = Well(
                well_id=wellbore.well_id,
                facility_name=wellbore.wellbore_name,
                marker_depths=[marker.depth for marker in sorted_markers],
                formations=[marker.name for marker in sorted_markers],
            )
            wells.append(well)
        return wells
