"""Module provides convenience wrappers for OSDU API and downloading helpers for Cloud Storage."""

import logging
from typing import Any, Dict, List, Optional

# import boto3
import requests
from deprecated import deprecated
# from azure.storage.blob.baseblobservice import BaseBlobService
from falcon import HTTP_BAD_REQUEST, HTTPError, Request

from src.settings import OSDU_API_FILE_URL, OSDU_API_SEARCH_URL

logger = logging.getLogger(__name__)

DEFAULT_QUERY_LIMIT = 99  # Limit the number of results to 99 if not explicitly set


def call_search_api(request: Request, search_query: Dict[str, Any]) -> Dict[str, Any]:
    """
    Prepare and make a request to OSDU search API.

    This function gets search query as a parameter, extracts session object
    from the request context and pulls auth tokens from the session. It then
    creates the required headers and makes a Search API call. Finally, it
    verifies the response status and passes the json results back to the caller.
    """
    # session object was added to the context by the authorization middleware
    # it contains both access_token and id_token required for the OSDU calls
    session = request.context.session
    logger.debug(
        f"Extracting authorization tokens from the session object: {session}")

    # create authorization headers
    headers = session.create_authorization_headers()
    if headers is None:
        raise HTTPError(
            HTTP_BAD_REQUEST,
            "Cannot make API call",
            "Unknown target platform, check OSDU_TARGET_PLATFORM setting."
        )

    # set the default pagination limit if not set explicitly in the query
    if "limit" not in search_query:
        search_query["limit"] = DEFAULT_QUERY_LIMIT

    # make request to OSDU Search API
    logger.debug(f"Starting API call to {OSDU_API_SEARCH_URL}")
    search_response = requests.post(
        OSDU_API_SEARCH_URL,
        headers=headers,
        json=search_query)
    logger.debug(f"Response details: {search_response.json()}")

    # verify status code
    if search_response.status_code != requests.codes.OK:
        logger.error(
            f"Request failed with the status code={search_response.status_code} \
            and message={search_response.text}")
        raise HTTPError(
            "{}".format(search_response.status_code),
            "Search API call failed",
            "OSDU Search API call has failed (HTTP {}): {}"
                .format(search_response.status_code, search_response.text))

    return search_response.json()


def get_signed_url(request: Request, file_id: str) -> str:
    """
    Prepare and make a request to OSDU File API.

    This function gets file id as a parameter, extracts session object
    from the request context and pulls auth tokens from the session. It then
    creates the required headers and makes a File API call. Finally, it
    verifies the response status and parses the json and returns Signed URL to the caller.
    """
    # session object was added to the context by the authorization middleware
    # it contains both access_token and id_token required for the OSDU calls
    session = request.context.session
    logger.debug(
        f"Extracting authorization tokens from session object: {session}")

    # create authorization headers
    headers = session.create_authorization_headers()
    if headers is None:
        raise HTTPError(
            HTTP_BAD_REQUEST,
            "Cannot make API call",
            "Unknown target platform, check OSDU_TARGET_PLATFORM setting."
        )

    # make request to OSDU File API
    logger.debug(f"Starting API call to {OSDU_API_FILE_URL}")

    file_response = requests.get(
        f"{OSDU_API_FILE_URL}/{file_id}/downloadURL",
        headers=headers
    )
    logger.debug(f"Response details: {file_response.json()}")

    # checks any errors after API call
    if file_response.status_code != requests.codes.OK:  # HTTP 200
        logger.error(
            f"Failed to call File API: {file_response.text}")
        raise HTTPError(
            "{}".format(file_response.status_code),
            "File API call failed",
            "OSDU File API call has failed (HTTP {}): {}"
                .format(file_response.status_code, file_response.text))

    # parse response and get Signed URL
    signed_url = file_response.json()["SignedUrl"]
    return signed_url


def download_files_from_cloud(signed_urls: List[str]) -> List[bytes]:
    """
    Download multiple files from the Cloud Storage using a list of signed URLs.

    The function calls cloud-neutral download_file_from_cloud function for each
    singed URL in the list and returns an array of files downloaded each represented
    as an array of bytes from the corresponding file.
    """
    files_data = []
    for file_info in signed_urls:
        files_data.append(download_file_by_url(file_info))

    return files_data


def download_file_by_url(file_info: str) -> Optional[bytes]:
    """Download a file using signed URL and return array of bytes."""
    try:
        blob_storage_response = requests.get(file_info)
    except KeyError as error:
        logger.error(f"Failed to extract signed URL.\
                         Missing attributes: {error}")
        return None

    if blob_storage_response.status_code != requests.codes.OK:  # HTTP 200
        logger.error(
            f"Failed to download data: {blob_storage_response.text}")
        return None

    return blob_storage_response.content


@deprecated(version="0.2.0", reason="You should use cloud-neutral\
    download_file_by_url() function")
def download_from_aws(file_info: Dict[str, Any]) -> Optional[bytes]:
    """
    Download a file from S3 storage and return an array of bytes.

    Function gets a list of results from Delivery API (signed URLs),
    then parses out the required parameters for boto3 downloader and
    returns the downloaded content. It uses AWS SDK (boto3) high-level
    functions, such as a resource.
    """
    # parse the response from Delivery API assuming only one (first) result
    try:
        key_name = file_info["StorageLocation"]["Key"]
        bucket_name = file_info["StorageLocation"]["Bucket"]
        kwargs = dict(
            aws_access_key_id=file_info["TemporaryCredentials"]["AccessKeyId"],
            aws_secret_access_key=file_info["TemporaryCredentials"]["SecretAccessKey"],
            aws_session_token=file_info["TemporaryCredentials"]["SessionToken"],
        )
    except KeyError as error:
        logger.error(f"Failed to extract parameters for S3 resource.\
                         Missing attributes: {error}")
        return None

    # download the file using AWS SDK (boto3)
    s3_resource = boto3.resource("s3", **kwargs)
    obj = s3_resource.Object(bucket_name, key_name)
    return obj.get()["Body"].read()


@deprecated(version="0.2.0", reason="You should use cloud-neutral\
    download_file_by_url() function")
def download_from_azure(file_info: Dict[str, Any]) -> Optional[bytes]:
    """
    Download a file from S3 storage and return an array of bytes.

    Function gets a list of results from Delivery API (signed URLs),
    then parses out the required parameters for az_blob
    downloader and returns the downloaded content. It uses Azure Blob
    library high-level function such as get_blob_to_text.
    """
    # parse the response from Delivery API assuming only one (first) result
    try:
        result_data: Dict = file_info["FileLocation"]
        endpoint = result_data["EndPoint"]
        bucket = result_data["Bucket"]
        key = result_data["Key"]
        sas = result_data["TemporaryCredentials"]["SAS"]
    except KeyError as error:
        logger.error(f"Failed to extract parameters for Azure Blob resource.\
                        Missing attributes: {error}")
        return None

    # download the file using Azure Blob lib
    service = BaseBlobService(
        account_name="Anonymous",
        sas_token=sas,
        custom_domain=endpoint)
    data = service.get_blob_to_text(bucket, key)
    return data.content
