"""Module implements searching wellbores by well ID."""
import json
import logging
from dataclasses import asdict, dataclass
from typing import Dict, List

from falcon import HTTP_NOT_FOUND, HTTPError, Request, Response

from src.settings import OSDU_DATA_PARTITION
from .utils import call_search_api

logger = logging.getLogger(__name__)


@dataclass
class Wellbore:
    """Oversimplified Wellbore structure."""

    wellbore_id: str  # Id of wellbore
    wellbore_name: str  # Wellbore Name (it is an Alias Name for wellbore)


@dataclass
class ResponseJSON:
    """Array of Wellbores in the Search Response."""

    wellbores: List[Wellbore]


class FindWellboresByWellIDResource:
    """Resource to find and process wellbores data (id, name)."""

    DEFAULT_ALIAS_NAME_TYPE_ID = f"{OSDU_DATA_PARTITION}:reference-data--AliasNameType:Borehole%20Code:"

    def on_get(self, request: Request, response: Response) -> None:
        """
        Find wellbores list by well_id.

        This handler gets well_id as a parameter and makes an OSDU Search API call
        to find master-data/wellbore objects by this well_id. If found, it returns
        the list of objects in the following format:
        {
            "wellbores":[
                {
                    "wellbore_id": str,
                    "wellbore_name": str
                }
            ]
        }
        """
        # get well_id from request, such as "opendes:master-data--Well:8690:"
        well_id = request.params.get("well_id")
        logger.debug(f"Preparing Search API call for well_id: {well_id}")

        # create Search API request payload:
        search_query = {
            "kind": f"{OSDU_DATA_PARTITION}:wks:master-data--Wellbore:1.0.0",
            "query": f"data.WellID: \"{well_id + ':'}\"",
            "returnedFields": [
                "id",
                "data.NameAliases"
            ]
        }
        logger.debug(f"Search request payload: {search_query}")

        response_data = call_search_api(request, search_query)

        # parse results and return back to frontend
        result_json = response_data.get("results", [])
        response.body = json.dumps(asdict(
            self.get_wellbores_from_search_response(result_json)))

    def get_wellbores_from_search_response(self, search_results: List[Dict]) -> ResponseJSON:
        """
        Parse and extract parameters from the search results.

        This function gets search results data and extracts the
        ResourceID and FacilityName attributes for each wellbore found.
        It returns a list of wellbores in the ResponseJSON object.
        """
        logger.debug("Parsing search results...")

        wellbores = []
        try:
            for result in search_results:
                wellbore = Wellbore(
                    wellbore_id=result["id"],
                    wellbore_name=self.extract_wellbore_name(result)
                )
                wellbores.append(wellbore)
        except KeyError as error:
            logger.exception(
                "Failed to extract parameters from Search API response")
            raise HTTPError(
                HTTP_NOT_FOUND,
                "Failed to extract parameters from Search API response",
                "Missing attributes: {}".format(error))

        logger.debug(f"Extracted wellbores from search results: {wellbores}")
        return ResponseJSON(wellbores=wellbores)

    def extract_wellbore_name(self, record: Dict) -> str:
        """
        Extract wellbore name from the record aliases.

        DEFAULT_ALIAS_NAME_TYPE_ID attribute value is used as a preferable alias type for wellbore name.
        """
        aliases = record["data"]["NameAliases"]

        for alias in aliases:
            if alias["AliasNameTypeID"] == self.DEFAULT_ALIAS_NAME_TYPE_ID:
                logger.debug(f"Found Wellbore name '{alias['AliasName']}'")
                return alias["AliasName"]
        else:
            logger.debug(
                f"No name for {self.DEFAULT_ALIAS_NAME_TYPE_ID:} alias in '{record}' record, "
                f"name '{aliases[0]['AliasName']}' will be used.")
            return aliases[0]["AliasName"]
