"""Module implements searching wells by name."""
import json
import logging
from dataclasses import asdict, dataclass
from typing import Dict, List, Tuple

from falcon import HTTP_NOT_FOUND, HTTPError, Request, Response

from src.settings import OSDU_DATA_PARTITION
from .utils import call_search_api

logger = logging.getLogger(__name__)


@dataclass
class GeoJSONPoint:
    """GeoJSON Point in response object."""

    coordinates: Tuple[float, float]
    type: str = "point"


@dataclass
class Well:
    """Well in response object."""

    well_id: str
    facility_id: str
    location: GeoJSONPoint


@dataclass
class ResponseJSON:
    """Response object."""

    wells: List[Well]


class FindWellsByNameResource:
    """Resource to find and process wells data (id, name)."""

    def on_get(self, request: Request, response: Response) -> None:
        """
        Find wells list by well_name.

        This handler gets the well name as a parameter and makes an OSDU Search API
        call to find master-data/well objects by this well name. If found, it returns
        the list of objects of the following format:
        {
            "wells":[
                {
                    "well_id": str
                    "facility_id": str,
                    "location": GeoJSONPoint
                }
            ]
        }
        """

        # get well name from request, such as "A05-01"
        well_name = request.params.get("well_name")

        logger.debug(f"Preparing Search API call for well name: {well_name}")

        # create Search API request payload:
        search_query = {
            "kind": f"{OSDU_DATA_PARTITION}:wks:master-data--Well:1.0.0",
            "query": f"data.FacilityID: \"{well_name}\"",
            "returnedFields": [
                "data.FacilityID",
                "id",
                "data.SpatialLocation.Wgs84Coordinates.geometries"
            ]
        }
        logger.debug(f"Search request payload: {search_query}")

        # response json
        search_response_json = call_search_api(request, search_query)

        # parse results and return back to frontend
        result_json = search_response_json.get("results", [])
        response.body = json.dumps(asdict(
            self.get_wells_from_search_response(result_json)))

    @staticmethod
    def get_wells_from_search_response(search_results: List[Dict]) -> ResponseJSON:
        """
        Parse and extract parameters from the search results.

        This function gets search results data and extracts
        FacilityID, id  attributes
        for each well found. It returns a list of wells in
        the ResponseJSON object.
        """
        logger.debug("Parsing search results...")
        wells = []
        try:
            for result in search_results:
                wells.append(Well(
                    well_id=result["id"],
                    facility_id=result["data"]["FacilityID"],
                    location=GeoJSONPoint(
                        result["data"]["SpatialLocation.Wgs84Coordinates"]["geometries"][0]["coordinates"])))
        except KeyError as error:
            logger.exception(
                "Failed to extract parameters from Search API response")
            raise HTTPError(
                HTTP_NOT_FOUND,
                "Failed to extract parameters from Search API response",
                "Missing attributes: {}".format(error))

        logger.debug(f"Extracted wells from search results: {wells}")
        return ResponseJSON(wells=wells)
