"""Module implements searching wells in a polygon."""
import logging
from dataclasses import asdict, dataclass
from json import dumps
from typing import Any, Dict, List, Optional, Tuple

import falcon

from src.settings import OSDU_DATA_PARTITION
from .utils import call_search_api

logger = logging.getLogger(__name__)

GeoJSONCoordinates = Tuple[float, float]


@dataclass
class GeoJSONPoint:
    """GeoJSON Point structure."""

    coordinates: GeoJSONCoordinates
    type: str = "point"


@dataclass
class GeoJSONPolygon:
    """GeoJSON Polygon structure."""

    def __post_init__(self):
        """Post-constructor validation (auto-executed after __init__)."""
        coordinates = self.coordinates

        if len(coordinates) != 1:
            raise TypeError(
                f"Coordinates list should have only one array."
                " You have {len(coordinates)}")

        if len(coordinates[0]) < 4:
            raise TypeError(
                f"Polygon can not be described in less than 4 coordinates."
                " You have {len(coordinates[0])}")

        if coordinates[0][0] != coordinates[0][-1]:
            raise TypeError(
                f"First and last coordinates should be equal for polygons."
                " {coordinates[0][0]} != {coordinates[0][-1]}")

        if self.type != "Polygon":
            raise TypeError(
                f"Field 'type' should be 'Polygon', found '{self.type}'")

    coordinates: List[List[GeoJSONCoordinates]]
    type: str = "polygon"


@dataclass
class OpenDESPoint:
    """Point structure as represented in OpenDES."""

    longitude: float
    latitude: float


@dataclass
class OpenDESPolygon:
    """Polygon structure as represented in OpenDES."""

    points: List[OpenDESPoint]

    def __post_init__(self):
        """Post-constructor validation (auto-executed after __init__)."""
        points = self.points
        if len(points) < 4:
            raise TypeError(
                f"Polygon can not be described in less than 4 points."
                f" You have {len(points)}")

        if points[0] != points[-1]:
            raise TypeError(
                f"First and last coordinates should be equal for polygon."
                f" {points[0]} != {points[-1]}")


@dataclass
class Well:
    """Oversimplified Well structure."""

    well_id: str
    facility_id: str
    location: GeoJSONPoint


class FindWellsInPolygonResource:
    """Resource to find and process wells data (id, name, location)."""

    def on_post(self, request: falcon.Request, response: falcon.Response) -> None:
        """
        Find wells list in a polygon.

        This handler gets a GeoJSON polygon as a parameter and makes an
        OSDU Search API call to find a master-data/well objects list.
        If found, it returns the list of objects in the following format:
        {
            "wells":[
                {
                    "well_id": <WellID>,
                    "facility_id": <FacilityId>,
                    "location": <GeoLocation>
                }
            ]
        }
        """
        # extract coordinates from geojson
        polygon = GeoJSONPolygon(**request.media)
        logger.debug(
            f"Preparing Search API call for the following polygon: {polygon.coordinates}")

        # create Search API request payload:
        # Note: OpenDES currently uses its own representation of coordinates,
        # so we need to transform GeoJSON -> OpenDES Points before making a Search call
        search_query = {
            "kind": f"{OSDU_DATA_PARTITION}:wks:master-data--Well:1.0.0",
            "spatialFilter": {
                "field": "data.SpatialLocation.Wgs84Coordinates",
                "byGeoPolygon": asdict(self.transform_geojson_to_opendes_polygon(polygon)),

            },
            "returnedFields": [
                "data.FacilityID",
                "id",
                "data.SpatialLocation.Wgs84Coordinates.geometries"
            ],
        }
        logger.debug(f"Search request payload: {search_query}")

        # response json
        results_json = call_search_api(request, search_query)

        # parse results and return back to frontend
        logger.debug(f"Results from Search API: {results_json.get('results')}")
        formatted_results = map(
            self.extract_well, results_json.get("results") or [])
        # filter out empty records
        filtered_results = filter(bool, formatted_results)
        response.body = dumps([asdict(well)
                               for well in filtered_results])

    @staticmethod
    def extract_well(result: Dict[str, Any]) -> Optional[Well]:
        """Extract wells data from raw results and format location as GeoJSON."""
        try:
            raw_location = result["data"]["SpatialLocation.Wgs84Coordinates"]["geometries"][0]["coordinates"]
            location = GeoJSONPoint(
                coordinates=raw_location)
            return Well(
                well_id=result["id"],
                facility_id=result["data"]["FacilityID"],
                location=location,
            )
        except KeyError:
            logger.exception(
                "Failed to extract parameters from search response")
            return None

    @staticmethod
    def transform_geojson_to_opendes_polygon(polygon: GeoJSONPolygon) -> OpenDESPolygon:
        """Transform Polygon coordinates from GeoJSON to OpenDES representation."""
        points = [OpenDESPoint(
            longitude=coord[0], latitude=coord[1]) for coord in polygon.coordinates[0]]
        return OpenDESPolygon(points=points)
