""" -----------------------------------------------------------------
 unittest module for fetch_markers_by_well_ids resource
 ----------------------------------------------------------------- """
import os
from json import loads
from unittest.mock import Mock, patch

from falcon import API, Request, testing

from src.fetch.fetch_markers_by_well_ids import FetchMarkersByWellIDsResource

CURRENT_DIR = os.path.join(os.getcwd(), os.path.dirname(__file__))

request_init = Request.__init__

MOCK_SCHEMA_VERSION = "1.0.0"
MOCK_OSDU_DATA_PARTITION = "opendes"

REQUEST_PARAMS_MOCK = {
    "well_ids": [
        f"{MOCK_OSDU_DATA_PARTITION}:master-data--Well:5093",
        f"{MOCK_OSDU_DATA_PARTITION}:master-data--Well:1149"
    ]
}


RESPONSE_JSON = [
    {
        "facility_name": "BIR-10",
        "formations": [
            "Upper North Sea Group",
            "Rupel Clay Member"
        ],
        "marker_depths": [
            0.0,
            480.0
        ],
        "well_id": f"{MOCK_OSDU_DATA_PARTITION}:master-data--Well:5093"
    },
    {
        "facility_name": "BIR-13",
        "formations": [
            "Upper North Sea Group",
            "Rupel Clay Member"
        ],
        "marker_depths": [
            0.0,
            480.0
        ],
        "well_id": f"{MOCK_OSDU_DATA_PARTITION}:master-data--Well:1149"
    }
]


def mocked_search_api(_request, query):
    """Mock request to OSDU Search API."""
    wellbore_query = f"\"{MOCK_OSDU_DATA_PARTITION}:master-data--Well:5093:\" OR \"{MOCK_OSDU_DATA_PARTITION}:master-data--Well:1149:\""
    find_wellbores = {
        "kind": f"{MOCK_OSDU_DATA_PARTITION}:wks:master-data--Wellbore:{MOCK_SCHEMA_VERSION}",
        "query": f"data.WellID: ({wellbore_query})",
        "returnedFields": ["id", "data.WellID", "data.NameAliases"]
    }

    marker_query = f"\"{MOCK_OSDU_DATA_PARTITION}:master-data--Wellbore:5093:\" OR \"{MOCK_OSDU_DATA_PARTITION}:master-data--Wellbore:1149:\""
    find_markers = {
        "kind": f"{MOCK_OSDU_DATA_PARTITION}:wks:work-product-component--WellboreMarkerSet:{MOCK_SCHEMA_VERSION}",
        "query": f"data.WellboreID: ({marker_query})",
        "returnedFields": ["data.WellboreID", "data.Markers"]
    }

    if query == find_wellbores:
        filename = "find_wellbores_r3_short.json"
    elif query == find_markers:
        filename = "find_markers_r3_short.json"
    else:
        raise Exception(f"Unknown query: {query}")

    return loads(get_file(filename))


def get_file(filename):
    """Get text content of a file."""
    path = os.path.join(CURRENT_DIR, filename)
    with open(path) as f:
        return f.read()


def mock_init(req, *args, **kwargs):
    """Put session into request context."""
    request_init(req, *args, **kwargs)
    req.context.session = Mock()


class TestFetchMarkersByWellIDsResource(testing.TestCase):
    """Test resource for fetching markers."""

    def setUp(self):
        """Create falcon server instance."""
        api = API()
        api.add_route("/fetch/fetch_markers_by_well_ids",
                      FetchMarkersByWellIDsResource())
        self.client = testing.TestClient(api)

    @patch("falcon.request.Request.__init__", mock_init)
    @patch("src.fetch.fetch_markers_by_well_ids.call_search_api", side_effect=mocked_search_api)
    @patch("src.fetch.fetch_markers_by_well_ids.OSDU_DATA_PARTITION", MOCK_OSDU_DATA_PARTITION)
    def test_fetch_markers_by_well_ids(self, *_args):
        """Test whole request with data from files. Check order of several markers."""

        link = "/fetch/fetch_markers_by_well_ids"
        result = self.client.simulate_get(link, params=REQUEST_PARAMS_MOCK)

        self.assertEqual(RESPONSE_JSON, result.json)
