""" -----------------------------------------------------------------
 unittest module for find_wells_by_name resource
 ----------------------------------------------------------------- """
from dataclasses import asdict
from unittest.mock import patch

from falcon import API, Request, testing

from src.find import FindWellsByNameResource
from src.find.find_wells_by_name import ResponseJSON

MOCK_WELL_NAME = "BIR-13"
MOCK_RESOURCE_ID = "opendes:master-data--Well:3687:"
MOCK_COORDINATES = {
    "coordinates": [
        6.88573846,
        53.3727582
    ],
    "type": "point"
}

OSDU_API_SEARCH_RESPONSE_MOCK = {
    "results": [
        {
            "data": {
                "FacilityID": MOCK_WELL_NAME,
                "SpatialLocation.Wgs84Coordinates": {
                    "geometries": [MOCK_COORDINATES]
                }
            },
            "id": MOCK_RESOURCE_ID
        }
    ],
    "aggregations": None,
    "totalCount": 1
}

RESPONSE_JSON = {
    "wells": [{
        "well_id": MOCK_RESOURCE_ID,
        "facility_id": MOCK_WELL_NAME,
        "location": MOCK_COORDINATES,
    }]
}

req_init = Request.__init__


def mock_falcon_request(req, *args, **kwargs):
    """
    Input params: req and args with env parameters and kwargs with request options.
    Function changes env parameters in falcon request for add mock well name
    """
    env = testing.create_environ(
        path=args[0]["PATH_INFO"],
        query_string="well_name={}".format(MOCK_WELL_NAME))
    req_init(req, env, **kwargs)


class BaseTestAPICase(testing.TestCase):
    """Base test API case which set up API and adds route"""

    def setUp(self):
        super(BaseTestAPICase, self).setUp()

        app = API()
        app.add_route("/find/wells_by_name", FindWellsByNameResource())

        self.app = app


class TestFindWellsByNameResource(BaseTestAPICase):
    """Checks test case situations in find_wells_by_name request handler."""

    @patch("src.find.find_wells_by_name.call_search_api")
    def test_find_wells_by_name_success(self, call_search_api_mock):
        """
        This test case checks success situation in find_wells_by_name request handler.
        """
        # mocks call_search_api function result
        call_search_api_mock.return_value = OSDU_API_SEARCH_RESPONSE_MOCK
        # simulates request with mock request parameters
        with patch("falcon.request.Request.__init__", mock_falcon_request):
            result = self.simulate_get("/find/wells_by_name")

            # checks response parameters
            self.assertEqual(result.status_code, 200)
            self.assertIsInstance(result.json, dict)
            self.assertEqual(result.json, RESPONSE_JSON)


class TestGetWellsFromSearchResponse(testing.TestCase):
    """Checks test case situations in get_wells_from_search_response function."""

    def test_get_wells_from_search_response_success(self):
        """
        This test case checks success situation in call get_wells_from_search_response function.
        """
        # mocks result json form search API
        result_json = OSDU_API_SEARCH_RESPONSE_MOCK["results"]

        # calls get_wells_from_search_response function
        resource = FindWellsByNameResource()
        response_media = resource.get_wells_from_search_response(result_json)

        # checks response parameters
        self.assertIsInstance(response_media, ResponseJSON)
        self.assertEqual(response_media.wells[0].well_id, MOCK_RESOURCE_ID)
        self.assertEqual(response_media.wells[0].facility_id, MOCK_WELL_NAME)
        self.assertEqual(asdict(response_media.wells[0].location), MOCK_COORDINATES)
