""" -----------------------------------------------------------------
 unittest module for find_wells_in_polygon resource
 ----------------------------------------------------------------- """

import json
import os
from unittest.mock import patch

import falcon
from falcon import API, testing

from src.find import FindWellsInPolygonResource

CURRENT_DIR = os.path.join(os.getcwd(), os.path.dirname(__file__))
RESPONSE_FILE_NAME = "search_api_response_polygon_search_r3_short.json"
PATH = os.path.join(CURRENT_DIR, RESPONSE_FILE_NAME)

request_init = falcon.request.Request.__init__

MOCK_OSDU_DATA_PARTITION = "opendes"
MOCK_GEO_POLYGON = {
    "type": "Polygon",
    "coordinates": [
        [
            [3.01, 52.38],
            [3.10, 52.13],
            [3.43, 52.21],
            [3.01, 52.38],
        ]
    ]
}

RESPONSE_JSON = [
    {
        "facility_id": "P13-02",
        "location": {
            "coordinates": [3.10862804, 52.26678235],
            "type": "point"
        },
        "well_id": "opendes:master-data--Well:7443"
    },
    {
        "facility_id": "P13-01",
        "location": {
            "coordinates": [3.19549275, 52.21519838],
            "type": "point"
        },
        "well_id": "opendes:master-data--Well:7442"
    }
]


class TestFindWellsInPolygonResource(testing.TestCase):
    """Base test API case which set up API and adds route"""

    def setUp(self):
        """Creating application with tested result"""
        api = API()
        api.add_route("/find/wells_in_polygon", FindWellsInPolygonResource())
        self.client = testing.TestClient(api)

    def mocked_call_search_api(self, _request, _query):
        """Mock for Response on post requests"""
        expected_query = {
            "kind": f"{MOCK_OSDU_DATA_PARTITION}:wks:master-data--Well:1.0.0",
            "returnedFields": [
                "data.FacilityID",
                "id",
                "data.SpatialLocation.Wgs84Coordinates.geometries"
            ],
            "spatialFilter": {
                "byGeoPolygon": {
                    "points": [
                        {
                            "latitude": 52.38,
                            "longitude": 3.01
                        },
                        {
                            "latitude": 52.13,
                            "longitude": 3.1
                        },
                        {
                            "latitude": 52.21,
                            "longitude": 3.43
                        },
                        {
                            "latitude": 52.38,
                            "longitude": 3.01
                        }
                    ]
                },
                "field": "data.SpatialLocation.Wgs84Coordinates"
            }
        }

        self.assertEqual(expected_query, _query)

        with open(PATH) as json_file:
            data = json.load(json_file)

        return data

    @patch("src.find.find_wells_in_polygon.OSDU_DATA_PARTITION", MOCK_OSDU_DATA_PARTITION)
    def test_find_wells_in_polygon(self):
        """Checks test case situations in find_wells_in_polygon request handler."""
        # request to remote server should be mocked
        with patch("src.find.find_wells_in_polygon.call_search_api",
                   side_effect=self.mocked_call_search_api):
            result = self.client.simulate_post(
                "/find/wells_in_polygon", json=MOCK_GEO_POLYGON)

        self.assertEqual(RESPONSE_JSON, result.json)
