# React-frontend

This is the frontend of the Quickstart application.

## How to run

1. Go to the `react-frontend` folder.

2. Install dependencies:

```
npm ci
```

3. Run the application:

```
npm start
```

## How to change the backend host and port

The `.env` file defines the following variables:

```
SERVER_HOST=backend
SERVER_PORT=8080
```
You can change them if needed.

Note: when using docker-compose to define and run both the UI and backend, the value of `SERVER_HOST` must be the same as the name of service for your backend:

```
services:
  backend:
    ...
  ui:
    ...
    depends_on:
      - backend
```

## Implemented user-interface features

- Searching for wells by name
- Searching for wells on map 
- Viewing wells on map
- Viewing the wellbores list of a well
- Plotting wellbore 3D trajectories
- Plotting a cross-section for a set of wells

## User stories

### Searching for wells by name

1. Open the main page.
2. In search field at the top left, type a well name. It will be treated as a part of the name, so no need for a wildcard.
3. Click **Search** and wait for the results. The list of up to 99 wells will be dispayed under the search bar. If the search failed, an error will be shown.

### Viewing the wellbores list of a well

1. Search for a well.
2. Click the well name or the angle-icon to run the wellbores fetching. The drop down list of wellbrores will be displayed under the well. If the fetch fails, an error will be shown.

### Plotting a 3D trajectory

1. Search for a well.
2. Fetch its wellbores.
3. Click the **VISUALIZE** button and wait for a while. A 3D model of the wellbore trajectory will be plotted. If the plotting fails, the button will change to ERROR; you can read the description in the tooltip.

It is possible to visualize multiple wellbores of a single well in the same 3D model.

### Viewing a well on map

1. Search for a well.
2. Click the **show on map** link next to the well name. The map will be loaded with the well in the center.

### Searching wells within a polygon on map

1. View a well on map.
2. Click on map to draw a polygon. Every click creates a new marker point. To enclose the polygon, click on the first point. You can edit a polygon:
  - Double click on a point to delete it.
  - Drag a point to change its location.
  - Drag a small auxiliary point to add a new point of the polygon.
3. Click the **Search polygon** icon and wait for the results. The first 99 found wells will be displayed as dots with their names appearing on hover. If an error occures, a popup with the message will be shown.

### Plotting a cross-section for a set of wells

1. Find wells within a polygon on map.
2. Choose several wells by clicking on their icons.
3. Click the **Create cross-section** icon at top right. You will be redirected to the cross-section page and the request to fetch data will be immediately sent. If something goes wrong, a descriptive message will be shown.

## Project composition and rationale

### create-react-app

The project is built on top of the create-react-app package as a unified method of starting small react projects.

- Designed to create such small projects
- It is popular and you can get your questions answered on Google
- Supported by the React Community

### TypeScript

We encourage all web developers to write a typed code as it is considered a good practice.

- TypeScript provides a comprehensive functionality for the subject.
- Widely used and supported.

### CSS

- The easiest way to write styles.
- The project is small enough not to deal with tough stuff.
- A perfect markup and styling are not the subject of the project.

### Redux

Our application has a quite complex inner state to use a state manager.

- Simple and descriptive idea.
- Fit for the project's small size.
- Redux is more popular than MobX.

### Thunk

As the application implements an asyncronous communication with the API, and responds to user actions iteratively, we need to handle side effects of Redux actions.

- Simple and descriptive idea.
- Fit for the project's small size.
- Easier to comprehend than the Redux Saga thunk.

### react-router

To build a modern single-page application, a route handler is required.

- The most popular router for React.

### Express.js

A small web server to deliver static files and perform API requests proxying.

- The de facto standart for such purposes at the moment.

### Jest

We encourage all web developers to cover a code with test.

- Delivered with create-react-app.
- Comprehensive documentation.
- Easy to use.

### Enzyme

We use it to enhance the react components testing.

- HTML snapshots are more descriptive than a virtual DOM structure.
- Enables inspecting component's children as thoroughly as we need.

## External libraries

### Recharts

- Used for the cross-section feature because a cross-section just looks like a linear stacked chart.
- Simple and popular JS library for drawing charts.

### Three

- Used for drawing trajectory 3D models.
- Popular JS library for working with 3D.

### Leaflet

- Used for searching wells on map
- Popular open-source library for interactive maps.

## Sequence diagrams, powered by Mermaid

[Mermaid Manual](https://mermaid-js.github.io/mermaid/#/sequenceDiagram)

### /api/find/wells_by_name

```mermaid
sequenceDiagram
  UI->>SearchButton: click
  SearchButton->>+Redux: find wells by name action
  Redux->>+Api: find wells by well name
  Api-x+Backend: request
  Backend--x-Api: response
  Api-->>-Redux: wells
  Redux-->>-UI: list of wells
```

### /api/find/wellbores_by_well_id

```mermaid
sequenceDiagram
  UI->>Well: click
  Well->>+Redux: set find wellbore action
  Redux->>+Api: find wellbores by well_id
  Api-x+Backend: request
  Backend--x-Api: response
  Api-->>-Redux: wellbores
  Redux-->>-UI: list of wellbores
```

### /api/fetch/trajectory_by_wellbore_id

```mermaid
sequenceDiagram
  UI->>Wellbore Visualise: click
  Wellbore Visualise->>+Redux: set visualize <br> trajectory action
  Redux->>+Api: fetch trajectory by wellbore_id
  Api-x+Backend: request
  Backend--x-Api: response
  Api-->>-Redux: trajectory data
  Redux-->>-UI: trajectory
```

### /login

```mermaid
sequenceDiagram
  UI->>LoginButton: click
  LoginButton-x+Backend: redirect to /login
  Backend--x-UI: redirect to / and set session cookie
```

### /logout

```mermaid
sequenceDiagram
  UI->>LogoutButton: click
  LogoutButton-x+Backend: redirect to /logout
  Backend--x-UI: redirect to / and clear session cookie
```

### /api/find/wells_in_polygon

```mermaid
sequenceDiagram
  UI->>SearchInPolygonButton: click
  SearchInPolygonButton->>+Redux: get points data action
  Redux->>+Api: find wells in polygon
  Api-x+Backend: request
  Backend--x-Api: response
  Api-->>-Redux: wells data
  Redux-->>-UI: wells
```

### /api/fetch/markers_by_well_ids

```mermaid
sequenceDiagram
  UI->>CrossSectionPage: navigate to
  CrossSectionPage->>+Redux: get markers action
  Redux->>+Api: fetch markers by well_ids
  Api-x+Backend: request
  Backend--x-Api: response
  Api-->>-Redux: cross section
  Redux-->>-UI: cross section
```
