import fetchMock from 'fetch-mock';
import configureMockStore from 'redux-mock-store';
import thunk, { ThunkDispatch } from 'redux-thunk';
import {
  loadWellsDataAction,
  loadWellsDataStartAction,
  loadWellsDataFailAction,
  loadWellsDataSuccessAction,
} from './actions';
import { crossSectionResponse } from 'test-data/cross-section';
import {
  CrossSectionActionTypes,
  LOAD_WELLS_DATA_START,
  LOAD_WELLS_DATA_SUCCESS,
  LOAD_WELLS_DATA_FAIL,
} from './types';
import { CrossSectionState } from './reducer';
import { AppState } from 'store';

const middlewares = [thunk];
const mockStore = configureMockStore<
  CrossSectionState,
  ThunkDispatch<AppState, null, CrossSectionActionTypes>
>(middlewares);

describe('Cross-section actions', () => {
  let store = mockStore();
  let action: CrossSectionActionTypes;
  let expectedActions: CrossSectionActionTypes[];

  beforeEach(() => {
    store = mockStore();
    fetchMock.restore();
  });

  describe('correct action objects creation', () => {
    it('creates LOAD_WELLS_DATA_START action', async () => {
      action = {
        type: LOAD_WELLS_DATA_START,
        payload: ['opendes:master-data--Well:1111', 'opendes:master-data--Well:2222'],
      };

      expect(loadWellsDataStartAction(['opendes:master-data--Well:1111', 'opendes:master-data--Well:2222'])).toEqual(action);
    });

    it('creates LOAD_WELLS_DATA_SUCCESS action', async () => {
      action = {
        type: LOAD_WELLS_DATA_SUCCESS,
        payload: crossSectionResponse,
      };

      expect(loadWellsDataSuccessAction(crossSectionResponse)).toEqual(action);
    });

    it('creates LOAD_WELLS_DATA_FAIL action', async () => {
      action = {
        type: LOAD_WELLS_DATA_FAIL,
        payload: Error('Forbidden'),
      };

      expect(loadWellsDataFailAction(Error('Forbidden'))).toEqual(action);
    });
  });

  describe('correct LOAD_WELLS_DATA actions sequence', () => {
    it('triggers START to SUCCESS when data is loaded', async () => {
      const mockWellIds: string[] = ['opendes:master-data/Well:1111', 'opendes:master-data/Well:2222'];
      fetchMock.getOnce(
        `/api/fetch/markers_by_well_ids?well_ids=${mockWellIds.join(',')}`,
        crossSectionResponse
      );

      expectedActions = [
        loadWellsDataStartAction(mockWellIds),
        loadWellsDataSuccessAction(crossSectionResponse),
      ];

      await store.dispatch(loadWellsDataAction(mockWellIds));
      expect(store.getActions()).toMatchObject(expectedActions);
    });

    it('triggers START to FAIL when data is unavailable', async () => {
      const mockWellIds: string[] = ['opendes:master-data/Well:1111', 'opendes:master-data/Well:2222'];
      fetchMock.getOnce(`/api/fetch/markers_by_well_ids?well_ids=${mockWellIds.join(',')}`, 403);

      expectedActions = [
        loadWellsDataStartAction(mockWellIds),
        loadWellsDataFailAction(Error('Forbidden')),
      ];

      await store.dispatch(loadWellsDataAction(mockWellIds));
      expect(store.getActions()).toMatchObject(expectedActions);
    });
  });
});
