import fetchMock from 'fetch-mock';
import configureMockStore from 'redux-mock-store';
import thunk, { ThunkDispatch } from 'redux-thunk';
import {
  findWellsAction,
  findWellsSuccessAction,
  determineCurrentUserPositionAction,
  findWellsStartAction,
  findWellsFailAction,
  setInitialWellAction,
  toggleWellSelectionAction,
  setUserPositionAction,
  finishDrawModeAction,
  addPointAction,
  changePointAction,
  deletePointAction,
  makePointRealAction,
  clearAllPointsAction,
  setMapZoomAction,
  setMapCenterAction,
  clearSearchError,
} from './actions';
import { LatLng } from 'leaflet';
import { fetchWellsInPolygonResponse, mapWithInitialWellState } from 'test-data/map';
import {
  MapActionTypes,
  SET_INITIAL_WELL,
  TOGGLE_WELL_SELECTION,
  SET_USER_POSITION,
  FINISH_DRAW_MODE,
  ADD_POINT,
  CHANGE_POINT,
  DELETE_POINT,
  MAKE_POINT_REAL,
  CLEAR_ALL_POINTS,
  FIND_WELLS_START,
  FIND_WELLS_SUCCESS,
  FIND_WELLS_FAIL,
  SET_MAP_ZOOM,
  SET_MAP_CENTER,
  CLEAR_SEARCH_ERROR,
} from './types';
import { MapState } from './reducer';
import { AppState } from 'store';

const middlewares = [thunk];
const mockStore = configureMockStore<MapState, ThunkDispatch<AppState, null, MapActionTypes>>(
  middlewares
);

describe('Map actions', () => {
  let store = mockStore();
  let action: MapActionTypes;
  let expectedActions: MapActionTypes[];

  beforeEach(() => {
    store = mockStore();
    fetchMock.restore();
  });

  describe('correct action objects creation', () => {
    it('creates SET_INITIAL_WELL action', async () => {
      action = {
        type: SET_INITIAL_WELL,
        payload: mapWithInitialWellState.initialWell,
      };

      expect(setInitialWellAction(mapWithInitialWellState.initialWell)).toEqual(action);
    });

    it('creates TOGGLE_WELL_SELECTION action', async () => {
      action = {
        type: TOGGLE_WELL_SELECTION,
        payload: 'opendes:master-data--Well:1111',
      };

      expect(toggleWellSelectionAction('opendes:master-data--Well:1111')).toEqual(action);
    });

    it('creates SET_USER_POSITION action', async () => {
      action = {
        type: SET_USER_POSITION,
        payload: new LatLng(1, 2),
      };

      expect(setUserPositionAction(new LatLng(1, 2))).toEqual(action);
    });

    it('creates FINISH_DRAW_MODE action', async () => {
      action = {
        type: FINISH_DRAW_MODE,
      };

      expect(finishDrawModeAction()).toEqual(action);
    });

    it('creates ADD_POINT action', async () => {
      action = {
        type: ADD_POINT,
        payload: new LatLng(1, 2),
      };

      expect(addPointAction(new LatLng(1, 2))).toEqual(action);
    });

    it('creates CHANGE_POINT action', async () => {
      action = {
        type: CHANGE_POINT,
        payload: {
          id: 'p0',
          position: new LatLng(1, 2),
        },
      };

      expect(changePointAction('p0', new LatLng(1, 2))).toEqual(action);
    });

    it('creates DELETE_POINT action', async () => {
      action = {
        type: DELETE_POINT,
        payload: 'p0',
      };

      expect(deletePointAction('p0')).toEqual(action);
    });

    it('creates MAKE_POINT_REAL action', async () => {
      action = {
        type: MAKE_POINT_REAL,
        payload: 'p0',
      };

      expect(makePointRealAction('p0')).toEqual(action);
    });

    it('creates CLEAR_ALL_POINTS action', async () => {
      action = {
        type: CLEAR_ALL_POINTS,
      };

      expect(clearAllPointsAction()).toEqual(action);
    });

    it('creates FIND_WELLS_START action', async () => {
      action = {
        type: FIND_WELLS_START,
      };

      expect(findWellsStartAction()).toEqual(action);
    });

    it('creates FIND_WELLS_SUCCESS action', async () => {
      action = {
        type: FIND_WELLS_SUCCESS,
        payload: fetchWellsInPolygonResponse,
      };

      expect(findWellsSuccessAction(fetchWellsInPolygonResponse)).toEqual(action);
    });

    it('creates FIND_WELLS_FAIL action', async () => {
      action = {
        type: FIND_WELLS_FAIL,
        payload: Error('Forbidden'),
      };

      expect(findWellsFailAction(Error('Forbidden'))).toEqual(action);
    });

    it('creates SET_MAP_ZOOM action', async () => {
      action = {
        type: SET_MAP_ZOOM,
        payload: 0,
      };

      expect(setMapZoomAction(0)).toEqual(action);
    });

    it('creates SET_MAP_CENTER action', async () => {
      action = {
        type: SET_MAP_CENTER,
        payload: new LatLng(1, 2),
      };

      expect(setMapCenterAction(new LatLng(1, 2))).toEqual(action);
    });

    it('creates CLEAR_SEARCH_ERROR action', async () => {
      action = {
        type: CLEAR_SEARCH_ERROR,
      };

      expect(clearSearchError()).toEqual(action);
    });
  });

  describe('correct FIND_WELLS actions sequence', () => {
    it('triggers START to SUCCESS when data is loaded', async () => {
      fetchMock.post('api/find/wells_in_polygon', fetchWellsInPolygonResponse);

      expectedActions = [
        findWellsStartAction(),
        findWellsSuccessAction(fetchWellsInPolygonResponse),
      ];

      await store.dispatch(findWellsAction([]));
      expect(store.getActions()).toMatchObject(expectedActions);
    });

    it('triggers START to FAIL when data is unavailable', async () => {
      fetchMock.post('api/find/wells_in_polygon', 403);

      expectedActions = [findWellsStartAction(), findWellsFailAction(Error('Forbidden'))];

      await store.dispatch(findWellsAction([]));
      expect(store.getActions()).toMatchObject(expectedActions);
    });
  });

  describe('correct SET_USER_POSITION actions sequence', () => {
    // as long as this is the only 'geolocation' definition in the file, everything is sound
    // different test files do not interfere in such a way
    Object.defineProperty(window.navigator, 'geolocation', {
      value: {
        getCurrentPosition: jest.fn(),
      },
    });

    it('triggers SET_USER_POSITION when access is granted', async () => {
      window.navigator.geolocation.getCurrentPosition = jest.fn(onSuccess =>
        Promise.resolve(
          onSuccess({
            coords: {
              latitude: 1,
              longitude: 2,
            },
            // Coordinates - is all we need in our action, therefore cautious type casting is acceptable
          } as Position)
        )
      );

      expectedActions = [setUserPositionAction(new LatLng(1, 2))];
      await store.dispatch(determineCurrentUserPositionAction());
      expect(store.getActions()).toEqual(expectedActions);
    });

    it('triggers nothing when request is declined', async () => {
      window.navigator.geolocation.getCurrentPosition = jest.fn((onSuccess, onError: () => void) =>
        Promise.resolve(onError())
      );

      expectedActions = [];
      await store.dispatch(determineCurrentUserPositionAction());
      expect(store.getActions()).toEqual(expectedActions);
    });
  });
});
