import { AppState } from 'store';
import { ThunkAction } from 'redux-thunk';
import { LatLng, Polygon } from 'leaflet';
import { Polygon as GeoPolygon } from 'geojson';
import {
  MapActionTypes,
  SET_USER_POSITION,
  FINISH_DRAW_MODE,
  ADD_POINT,
  CHANGE_POINT,
  DELETE_POINT,
  FIND_WELLS_SUCCESS,
  FIND_WELLS_START,
  FIND_WELLS_FAIL,
  TOGGLE_WELL_SELECTION,
  SET_INITIAL_WELL,
  SET_MAP_ZOOM,
  SET_MAP_CENTER,
  CLEAR_ALL_POINTS,
  MAKE_POINT_REAL,
  CLEAR_SEARCH_ERROR,
} from './types';
import { fetchWellsInPolygon, FetchWellsInPolygonResponse } from 'api/map.api';
import { Well, PolygonPoint } from './reducer';

type AppThunk<ReturnType = void> = ThunkAction<ReturnType, AppState, null, MapActionTypes>;

export function setInitialWellAction(well?: Well): MapActionTypes {
  return {
    type: SET_INITIAL_WELL,
    payload: well,
  };
}

/**
 * toggle well to be included into a cross-section report
 * @param wellId
 */
export function toggleWellSelectionAction(wellId: string): MapActionTypes {
  return {
    type: TOGGLE_WELL_SELECTION,
    payload: wellId,
  };
}

export function setUserPositionAction(position: LatLng): MapActionTypes {
  return {
    type: SET_USER_POSITION,
    payload: position,
  };
}

/**
 * comand to finish a draw mode
 * and to enclose a polygon with a given set of points
 */
export function finishDrawModeAction(): MapActionTypes {
  return {
    type: FINISH_DRAW_MODE,
  };
}

/**
 * add a new point to a polygon
 * @param position
 */
export function addPointAction(position: LatLng): MapActionTypes {
  return {
    type: ADD_POINT,
    payload: position,
  };
}

/**
 * change a position of a polygon point
 * @param id
 * @param position a new geo position
 */
export function changePointAction(id: string, position: LatLng): MapActionTypes {
  return {
    type: CHANGE_POINT,
    payload: {
      id,
      position: position,
    },
  };
}

/**
 * remove a single polygon point
 * @param id
 */
export function deletePointAction(id: string): MapActionTypes {
  return {
    type: DELETE_POINT,
    payload: id,
  };
}

/** comand to convert an 'edit' point to a real one */
export function makePointRealAction(id: string): MapActionTypes {
  return {
    type: MAKE_POINT_REAL,
    payload: id,
  };
}

/** remove any polygon points and start its drawing anew */
export function clearAllPointsAction(): MapActionTypes {
  return {
    type: CLEAR_ALL_POINTS,
  };
}

export function findWellsStartAction(): MapActionTypes {
  return {
    type: FIND_WELLS_START,
  };
}

export function findWellsSuccessAction(wells: FetchWellsInPolygonResponse): MapActionTypes {
  return {
    type: FIND_WELLS_SUCCESS,
    payload: wells,
  };
}

export function findWellsFailAction(err: Error): MapActionTypes {
  return {
    type: FIND_WELLS_FAIL,
    payload: err,
  };
}

/**
 * This will keep the state synced with a map zoom,
 * changed by user's interactions with a leaflet
 * @param zoom a new map zoom value
 */
export function setMapZoomAction(zoom: number): MapActionTypes {
  return {
    type: SET_MAP_ZOOM,
    payload: zoom,
  };
}

/**
 * This will keep the state synced with a map postition
 * changed by user's interactions with a leaflet
 * @param center a new map center coordinates
 */
export function setMapCenterAction(center: LatLng): MapActionTypes {
  return {
    type: SET_MAP_CENTER,
    payload: center,
  };
}

/**
 * mark that a user acknowledged and error an it should be hidden now
 */
export function clearSearchError(): MapActionTypes {
  return {
    type: CLEAR_SEARCH_ERROR,
  };
}

/** try to determine user's position */
export const determineCurrentUserPositionAction = (): AppThunk => dispatch => {
  return new Promise(resolve => {
    window.navigator.geolocation.getCurrentPosition(
      position => {
        const userPosition = new LatLng(position.coords.latitude, position.coords.longitude);
        dispatch(setUserPositionAction(userPosition));
        resolve();
      },
      error => {
        // User position haven't got. Just get over it, you know.
        resolve();
      }
    );
  });
};

/**
 * Fetch from the backend a list of wells, located within a given polygon, to draw on the map
 * @param points a set of geo point, forming a valid polygon
 */
export const findWellsAction = (points: PolygonPoint[]): AppThunk => async dispatch => {
  dispatch(findWellsStartAction());
  const geoPints: LatLng[] = points.filter(p => p.type === 'real').map(p => p.position);
  // the type is Polygon | MultiPolygon, but we do not work with MultiPolygon
  const geometry = new Polygon(geoPints).toGeoJSON().geometry as GeoPolygon;

  return fetchWellsInPolygon(geometry)
    .then(data => dispatch(findWellsSuccessAction(data)))
    .catch((err: Error) => dispatch(findWellsFailAction(err)));
};
